﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Common;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Data;

namespace HeuristicLab.Problems.Scheduling.Encodings.PermutationWithRepetition {
  [Item("PWRDecoder", "An item used to convert a PWR-individual into a generalized schedule.")]
  [StorableClass]
  public class PWRDecoder : JSSPDecoder<PWREncoding>, IStochasticOperator {
    [StorableConstructor]
    protected PWRDecoder(bool deserializing) : base(deserializing) { }
    protected PWRDecoder(PWRDecoder original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new PWRDecoder(this, cloner);
    }

    public ILookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }

    public PWRDecoder()
      : base() {
      Parameters.Add(new LookupParameter<IRandom>("Random", "The pseudo random number generator which should be used for stochastic manipulation operators."));
    }

    public override Schedule CreateScheduleFromEncoding(PWREncoding solution) {
      ItemList<JSSPJob> jobs = (ItemList<JSSPJob>)Jobs.Clone();
      Schedule resultingSchedule = new Schedule(new IntValue(Jobs[0].Tasks.Count));
      foreach (int jobNr in solution.PermutationWithRepetition) {
        int i = 0;
        while (jobs[jobNr].Tasks[i].IsScheduled.Value) i++;
        jobs[jobNr].Tasks[i].IsScheduled.Value = true;
        resultingSchedule.AddTask(jobs[jobNr].Tasks[i], jobs[jobNr]);
      }
      return resultingSchedule;
    }

    public override IOperation Apply() {
      return base.Apply();
    } 
  }
}
